import { Channels } from "@/models/index.js";
import { makePaginationQuery } from "../../common/make-pagination-query.js";
import define from "../../define.js";

export const meta = {
	tags: ["channels", "account"],

	requireCredential: true,

	kind: "read:channels",

	res: {
		type: "array",
		optional: false,
		nullable: false,
		items: {
			type: "object",
			optional: false,
			nullable: false,
			ref: "Channel",
		},
	},
} as const;

export const paramDef = {
	type: "object",
	properties: {
		sinceId: { type: "string", format: "misskey:id" },
		untilId: { type: "string", format: "misskey:id" },
		limit: { type: "integer", minimum: 1, maximum: 100, default: 5 },
	},
	required: [],
} as const;

export default define(meta, paramDef, async (ps, me) => {
	const query = makePaginationQuery(
		Channels.createQueryBuilder(),
		ps.sinceId,
		ps.untilId,
	).andWhere({ userId: me.id });

	const channels = await query.take(ps.limit).getMany();

	return await Promise.all(channels.map((x) => Channels.pack(x, me)));
});
