import { ClipNotes, Clips } from "@/models/index.js";
import { getNote } from "../../common/getters.js";
import define from "../../define.js";
import { ApiError } from "../../error.js";

export const meta = {
	tags: ["account", "notes", "clips"],

	requireCredential: true,

	kind: "write:account",

	errors: {
		noSuchClip: {
			message: "No such clip.",
			code: "NO_SUCH_CLIP",
			id: "b80525c6-97f7-49d7-a42d-ebccd49cfd52",
		},

		noSuchNote: {
			message: "No such note.",
			code: "NO_SUCH_NOTE",
			id: "aff017de-190e-434b-893e-33a9ff5049d8",
		},
	},
} as const;

export const paramDef = {
	type: "object",
	properties: {
		clipId: { type: "string", format: "misskey:id" },
		noteId: { type: "string", format: "misskey:id" },
	},
	required: ["clipId", "noteId"],
} as const;

export default define(meta, paramDef, async (ps, user) => {
	const clip = await Clips.findOneBy({
		id: ps.clipId,
		userId: user.id,
	});

	if (clip == null) {
		throw new ApiError(meta.errors.noSuchClip);
	}

	const note = await getNote(ps.noteId).catch((e) => {
		if (e.id === "9725d0ce-ba28-4dde-95a7-2cbb2c15de24")
			throw new ApiError(meta.errors.noSuchNote);
		throw e;
	});

	await ClipNotes.delete({
		noteId: note.id,
		clipId: clip.id,
	});
});
