import { DriveFolders } from "@/models/index.js";
import { makePaginationQuery } from "../../common/make-pagination-query.js";
import define from "../../define.js";

export const meta = {
	tags: ["drive"],

	requireCredential: true,

	kind: "read:drive",

	res: {
		type: "array",
		optional: false,
		nullable: false,
		items: {
			type: "object",
			optional: false,
			nullable: false,
			ref: "DriveFolder",
		},
	},
} as const;

export const paramDef = {
	type: "object",
	properties: {
		limit: { type: "integer", minimum: 1, maximum: 100, default: 10 },
		sinceId: { type: "string", format: "misskey:id" },
		untilId: { type: "string", format: "misskey:id" },
		folderId: {
			type: "string",
			format: "misskey:id",
			nullable: true,
			default: null,
		},
	},
	required: [],
} as const;

export default define(meta, paramDef, async (ps, user) => {
	const query = makePaginationQuery(
		DriveFolders.createQueryBuilder("folder"),
		ps.sinceId,
		ps.untilId,
	).andWhere("folder.userId = :userId", { userId: user.id });

	if (ps.folderId) {
		query.andWhere("folder.parentId = :parentId", { parentId: ps.folderId });
	} else {
		query.andWhere("folder.parentId IS NULL");
	}

	const folders = await query.take(ps.limit).getMany();

	return await Promise.all(folders.map((folder) => DriveFolders.pack(folder)));
});
