import { Followings } from "@/models/index.js";
import { makePaginationQuery } from "../../common/make-pagination-query.js";
import define from "../../define.js";

export const meta = {
	tags: ["federation"],

	requireCredential: true,
	requireAdmin: true,
	requireCredentialPrivateMode: true,

	res: {
		type: "array",
		optional: false,
		nullable: false,
		items: {
			type: "object",
			optional: false,
			nullable: false,
			ref: "Following",
		},
	},
} as const;

export const paramDef = {
	type: "object",
	properties: {
		host: { type: "string" },
		sinceId: { type: "string", format: "misskey:id" },
		untilId: { type: "string", format: "misskey:id" },
		limit: { type: "integer", minimum: 1, maximum: 100, default: 10 },
	},
	required: ["host"],
} as const;

export default define(meta, paramDef, async (ps, me) => {
	const query = makePaginationQuery(
		Followings.createQueryBuilder("following"),
		ps.sinceId,
		ps.untilId,
	).andWhere("following.followerHost = :host", { host: ps.host });

	const followings = await query.take(ps.limit).getMany();

	return await Followings.packMany(followings, me, { populateFollowee: true });
});
