import { HOUR } from "@/const.js";
import { DriveFiles } from "@/models/index.js";
import { createImportFollowingJob } from "@/queue/index.js";
import define from "../../define.js";
import { ApiError } from "../../error.js";

export const meta = {
	secure: true,
	requireCredential: true,
	limit: {
		duratition: HOUR,
		max: 1,
	},

	errors: {
		noSuchFile: {
			message: "No such file.",
			code: "NO_SUCH_FILE",
			id: "b98644cf-a5ac-4277-a502-0b8054a709a3",
		},

		unexpectedFileType: {
			message: "Must be a CSV or JSON file.",
			code: "UNEXPECTED_FILE_TYPE",
			id: "660f3599-bce0-4f95-9dde-311fd841c183",
		},

		tooBigFile: {
			message: "That file is too big.",
			code: "TOO_BIG_FILE",
			id: "dee9d4ed-ad07-43ed-8b34-b2856398bc60",
		},

		emptyFile: {
			message: "That file is empty.",
			code: "EMPTY_FILE",
			id: "31a1b42c-06f7-42ae-8a38-a661c5c9f691",
		},
	},
} as const;

export const paramDef = {
	type: "object",
	properties: {
		fileId: { type: "string", format: "misskey:id" },
	},
	required: ["fileId"],
} as const;

export default define(meta, paramDef, async (ps, user) => {
	const file = await DriveFiles.findOneBy({ id: ps.fileId });

	if (file == null) throw new ApiError(meta.errors.noSuchFile);
	//if (!file.type.endsWith('/csv')) throw new ApiError(meta.errors.unexpectedFileType);
	if (file.size > 2_000_000) throw new ApiError(meta.errors.tooBigFile);
	if (file.size === 0) throw new ApiError(meta.errors.emptyFile);

	createImportFollowingJob(user, file.id);
});
