import { Users } from "@/models/index.js";
import { removePinned } from "@/services/i/pin.js";
import define from "../../define.js";
import { ApiError } from "../../error.js";

export const meta = {
	tags: ["account", "notes"],

	requireCredential: true,

	kind: "write:account",

	errors: {
		noSuchNote: {
			message: "No such note.",
			code: "NO_SUCH_NOTE",
			id: "454170ce-9d63-4a43-9da1-ea10afe81e21",
		},
	},

	res: {
		type: "object",
		optional: false,
		nullable: false,
		ref: "MeDetailed",
	},
} as const;

export const paramDef = {
	type: "object",
	properties: {
		noteId: { type: "string", format: "misskey:id" },
	},
	required: ["noteId"],
} as const;

export default define(meta, paramDef, async (ps, user) => {
	await removePinned(user, ps.noteId).catch((e) => {
		if (e.id === "b302d4cf-c050-400a-bbb3-be208681f40c")
			throw new ApiError(meta.errors.noSuchNote);
		throw e;
	});

	return await Users.pack<true, true>(user.id, user, {
		detail: true,
	});
});
